@@ -1,36 +1,20 @@
 #!/usr/bin/perl -w
 
+#use lib '../blib/lib', '../blib/arch';
+
 ### calculate_precision_and_recall_from_file_based_relevancies_for_LSA.pl
 
-#use lib '../blib/lib', '../blib/arch';
+##  This script looks for human-supplied relevancy judgments in a file called
+##  `relevancy.txt' for the queries in the file `test_queries.txt'
+
+##  See Item 10 of the README of the `examples' directory for further information.
+
 
 use strict;
 use Algorithm::VSM;
 
 my $corpus_dir = "corpus";                     # This is the directory containing
                                                # the corpus
-#my $corpus_dir = "corpus_with_java_and_cpp";
-#my $corpus_dir = "minicorpus";
-#my $corpus_dir = "microcorpus";
-
-my $corpus_vocab_db = "corpus_vocab_db";       # The corpus-wide histogram of the
-                                               # is stored in this DBM file for
-                                               # future use if so needed.
-
-my $doc_vectors_db  = "doc_vectors_db";        # Using the Storable module, we
-                                               # store all the doc vectors in 
-                                               # this diskfile in case the user
-                                               # would want to use vectors 
-                                               # directly off the disk.
-
-my $normalized_doc_vecs_db = "normalized_doc_vecs_db";
-                                               # Using the Storable module, we
-                                               # store the normalized vectors
-                                               # for the document in this database.
-                                               # Normalization means replacing the
-                                               # term occurrence counts by relative 
-                                               # frequencies and multiplying by
-                                               # idf(t) 
 
 my $stop_words_file = "stop_words.txt";        # Will typically include the 
                                                # keywords of the programming
@@ -46,12 +30,10 @@ my $relevancy_file   = "relevancy.txt";        # The humans-supplied relevancies
 
 my $lsa = Algorithm::VSM->new( 
                    corpus_directory    => $corpus_dir,
-                   corpus_vocab_db     => $corpus_vocab_db,
-                   doc_vectors_db      => $doc_vectors_db,
-                   normalized_doc_vecs_db   => $normalized_doc_vecs_db,
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
+                   min_word_length     => 4,
                    break_camelcased_and_underscored  => 1,  #default is 1
                    lsa_svd_threshold   => 0.01,     # Used for rejecting singular
                                                     # values that are smaller than
