@@ -1,27 +1,19 @@
 #!/usr/bin/perl -w
 
+#use lib '../blib/lib', '../blib/arch';
+
 ### calculate_precision_and_recall_from_file_based_relevancies_for_VSM.pl
 
-#use lib '../blib/lib', '../blib/arch';
+##  This script looks for human-supplied relevancy judgments in a file called
+##  `relevancy.txt' for the queries in the file `test_queries.txt'
+
+##  See Item 9 of the README of the `examples' directory for further information.
 
 use strict;
 use Algorithm::VSM;
 
 my $corpus_dir = "corpus";                     # This is the directory containing
                                                # the corpus
-#my $corpus_dir = "corpus_with_java_and_cpp";
-#my $corpus_dir = "minicorpus";
-#my $corpus_dir = "microcorpus";
-
-my $corpus_vocab_db = "corpus_vocab_db";       # The corpus-wide histogram of the
-                                               # is stored in this DBM file for
-                                               # future use if so needed.
-
-my $doc_vectors_db  = "doc_vectors_db";        # Using the Storable module, we
-                                               # store all the doc vectors in 
-                                               # this disk file in case the user
-                                               # would want to use vectors 
-                                               # directly off the disk.
 
 my $stop_words_file = "stop_words.txt";        # Will typically include the 
                                                # keywords of the programming
@@ -37,12 +29,11 @@ my $relevancy_file   = "relevancy.txt";        # The humans-supplied relevancies
 
 my $vsm = Algorithm::VSM->new( 
                    corpus_directory    => $corpus_dir,
-                   corpus_vocab_db     => $corpus_vocab_db,
-                   doc_vectors_db      => $doc_vectors_db,
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
                    break_camelcased_and_underscored  => 1,  #default is 1
+                   min_word_length     => 4,
                    relevancy_file      => $relevancy_file,
           );
 
