@@ -1,17 +1,19 @@
 #!/usr/bin/perl -w
 
-### calculate_similarity_matrix_for_all_docs.pl
-
 #use lib '../blib/lib', '../blib/arch';
 
-# This script demonstrates how you can calculate the similarity matrix for
-# all of the documents in your corpus.  The (i,j)th element of the output
-# matrix is the dot-vector based similarity between the i-th document and
-# the j-th document.  The index associated with a documnet is its place in
-# an alphabetically sorted list of all the documents.
+### calculate_similarity_matrix_for_all_docs.pl
 
-# This scirpt compares documents using their regular vector
-# representations.
+#  This script demonstrates how you can calculate the similarity matrix for
+#  all of the documents in your corpus.  The (i,j)th element of the output
+#  matrix is the dot-vector based similarity between the i-th document and
+#  the j-th document.  The index associated with a documnet is its place in
+#  an alphabetically sorted list of all the documents.
+
+#  This scirpt compares documents using their regular vector
+#  representations.
+
+#  See Item 12 of README of the `examples' directory
 
 
 use strict;
@@ -24,6 +26,7 @@ my $stop_words_file = "stop_words.txt";
 my $vsm = Algorithm::VSM->new( 
                    corpus_directory         => $corpus_dir,
                    stop_words_file          => $stop_words_file,
+                   min_word_length          => 4,
                    want_stemming            => 1, 
                    break_camelcased_and_underscored  => 1,  #default is 1
           );
