@@ -1,8 +1,10 @@
 #!/usr/bin/perl -w
 
+#use lib '../blib/lib', '../blib/arch';
+
 ### significance_testing.pl
 
-#use lib '../blib/lib', '../blib/arch';
+##  See Item 11 in the README of the `examples' directory.
 
 use strict;
 use Algorithm::VSM;
@@ -28,9 +30,6 @@ my $corpus_dir = "corpus";
 
 my $query_file      = "test_queries.txt";
 my $stop_words_file = "stop_words.txt";  
-my $corpus_vocab_db = "corpus_vocab_db";
-my $doc_vectors_db  = "doc_vectors_db";
-my $normalized_doc_vecs_db = "normalized_doc_vecs_db";
 my $relevancy_file   = "relevancy.txt"; 
 
 
@@ -46,15 +45,13 @@ my $relevancy_file   = "relevancy.txt";
 
 my $lsa1 = Algorithm::VSM->new( 
                    corpus_directory    => $corpus_dir,
-                   corpus_vocab_db     => $corpus_vocab_db,
-                   doc_vectors_db      => $doc_vectors_db,
-                   normalized_doc_vecs_db   => $normalized_doc_vecs_db,
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
                    break_camelcased_and_underscored  => 1,  #default is 1
                    lsa_svd_threshold   => $THRESHOLD_1,
                    relevancy_file      => $relevancy_file,
+                   min_word_length     => 4,
           );
 
 $lsa1->get_corpus_vocabulary_and_word_counts();
@@ -75,15 +72,12 @@ print "Avg precisions for LSA-1: @$avg_precisions_1\n"
 
 my $lsa2 = Algorithm::VSM->new( 
                    corpus_directory    => $corpus_dir,
-                   corpus_vocab_db     => $corpus_vocab_db,
-                   doc_vectors_db      => $doc_vectors_db,
-                   normalized_doc_vecs_db   => $normalized_doc_vecs_db,
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
                    lsa_svd_threshold   => $THRESHOLD_2,
                    relevancy_file      => $relevancy_file,
-#                   debug               => 1,
+                   min_word_length     => 4,
           );
 
 $lsa2->get_corpus_vocabulary_and_word_counts();
