@@ -1,7 +1,7 @@
 package Algorithm::VSM;
 
 #---------------------------------------------------------------------------
-# Copyright (c) 2014 Avinash Kak. All rights reserved.  This program is
+# Copyright (c) 2015 Avinash Kak. All rights reserved.  This program is
 # free software.  You may modify and/or distribute it under the same terms
 # as Perl itself.  This copyright notice must remain attached to the file.
 #
@@ -23,11 +23,10 @@ use Fcntl;
 use Storable;
 use Cwd;
 
-our $VERSION = '1.50';
+our $VERSION = '1.60';
 
-# from camelcase splits from perlmonks:
-my $_regex = 
-           qr/[[:lower:]0-9]+|[[:upper:]0-9](?:[[:upper:]0-9]+|[[:lower:]0-9]*)(?=$|[[:upper:]0-9])/; 
+# for camelcase splits (from perlmonks):
+my $_regex = qr/[[:lower:]0-9]+|[[:upper:]0-9](?:[[:upper:]0-9]+|[[:lower:]0-9]*)(?=$|[[:upper:]0-9])/; 
 
 ###################################   Constructor  #######################################
 
@@ -41,17 +40,13 @@ sub new {
           "--- perhaps a misspelling\n" 
           if _check_for_illegal_params(@params) == 0;
     bless {
-        _corpus_directory       =>  $args{corpus_directory} 
-                                        || "",
-        _save_model_on_disk     =>  exists $args{save_model_on_disk} ?
-                                          $args{save_model_on_disk} : 1,
+        _corpus_directory       =>  $args{corpus_directory}   || "",
+        _save_model_on_disk     =>  $args{save_model_on_disk} || 0,
         _break_camelcased_and_underscored  => exists $args{break_camelcased_and_underscored} ?
                                               $args{break_camelcased_and_underscored} : 1,
-        _corpus_vocab_db        =>  $args{corpus_vocab_db} 
-                                        || "corpus_vocab_db",
+        _corpus_vocab_db        =>  $args{corpus_vocab_db} || "corpus_vocab_db",
         _doc_vectors_db         =>  $args{doc_vectors_db} || "doc_vectors_db",
-        _normalized_doc_vecs_db =>  $args{normalized_doc_vecs_db} 
-                                        || "normalized_doc_vecs_db",
+        _normalized_doc_vecs_db =>  $args{normalized_doc_vecs_db} || "normalized_doc_vecs_db",
         _stop_words_file        =>  $args{stop_words_file} || "",
         _query_file             =>  $args{query_file} || "",
         _min_word_length        =>  $args{min_word_length} || 4,
@@ -60,7 +55,8 @@ sub new {
                                           $args{use_idf_filter} : 1,
         _max_number_retrievals  =>  $args{max_number_retrievals} || 30,
         _lsa_svd_threshold      =>  $args{lsa_svd_threshold} || 0.01,
-        _relevancy_threshold    =>  $args{relevancy_threshold} || 1,
+        _relevancy_threshold    =>  exists $args{relevancy_threshold} ?
+                                                   $args{relevancy_threshold} : 1,
         _relevancy_file         =>  $args{relevancy_file} || "",
         _debug                  =>  $args{debug} || 0,
         _working_directory      =>  cwd,
@@ -331,7 +327,7 @@ sub retrieve_with_vsm {
     }
     my @query_word_counts = values %{$self->{_query_vector}};
     my $query_word_count_total = reduce(\@query_word_counts);
-    die "Query does not contain corpus words. Nothing retrieved."
+    die "\nYour query does not contain corpus words. Nothing retrieved.\n"
         unless $query_word_count_total;
     my %retrievals;
     if ($self->{_idf_filter_option}) {
@@ -1186,9 +1182,9 @@ sub get_integer_suffix {
 
 =head1 NAME
 
-Algorithm::VSM --- A Perl module for retrieving files and documents from a
-software library with the VSM (Vector Space Model) and LSA (Latent Semantic
-Analysis) algorithms in response to search words.
+Algorithm::VSM --- A Perl module for retrieving files and documents from a software
+library with the VSM (Vector Space Model) and LSA (Latent Semantic Analysis)
+algorithms in response to search words and phrases.
 
 =head1 SYNOPSIS
 
@@ -1199,19 +1195,14 @@ Analysis) algorithms in response to search words.
         my $corpus_dir = "corpus";
         my @query = qw/ program ListIterator add ArrayList args /;
         my $stop_words_file = "stop_words.txt";  
-        my $corpus_vocab_db = "corpus_vocab_db";
-        my $doc_vectors_db  = "doc_vectors_db"; 
-        my $normalized_doc_vecs_db  = "normalized_doc_vecs_db";
         my $vsm = Algorithm::VSM->new( 
                            corpus_directory         => $corpus_dir,
-                           corpus_vocab_db          => $corpus_vocab_db,
-                           doc_vectors_db           => $doc_vectors_db, 
-                           normalized_doc_vecs_db   => $normalized_doc_vecs_db,
+                           use_idf_filter           => 1,
                            stop_words_file          => $stop_words_file,
-                           save_model_on_disk       => 0,  
-                           break_camelcased_and_underscored  => 1, 
                            max_number_retrievals    => 10,
                            want_stemming            => 1,  
+                           min_word_length          => 4,
+                           break_camelcased_and_underscored  => 1, 
         );
         $vsm->get_corpus_vocabulary_and_word_counts();
         $vsm->display_corpus_vocab();
@@ -1224,31 +1215,25 @@ Analysis) algorithms in response to search words.
         my $retrievals = $vsm->retrieve_for_query_with_vsm( \@query );
         $vsm->display_retrievals( $retrievals );
 
-     The constructor parameter 'corpus_directory' is for naming the root of
-     the directory whose VSM model you wish to construct.  The optional
-     parameters 'corpus_vocab_db', 'doc_vectors_db', and
-     'normalized_doc_vecs_db' are for naming disk-based databases in which
-     the VSM model will be stored.  Subsequently, these databases can be
-     used for much faster retrieval from the same corpus.  You need to set
-     these parameters only if you have set the save_model_on_disk option in
-     the constructor call.  The parameter 'want_stemming' means that you
-     would want the words in the documents to be stemmed to their root
-     forms before the VSM model is constructed.  Stemming will reduce all
-     words such as 'programming,' 'programs,' 'program,' etc. to the same
-     root word 'program.'  The parameter break_camelcased_and_underscored
-     when set will cause all underscored and camel-cased words to be split.
-     The functions display_corpus_vocab() and display_doc_vectors() are
-     there only for testing purposes with small corpora.  If you must use
-     them for large libraries/corpora, you might wish to redirect the
-     output to a file.
+     The constructor parameter 'corpus_directory' is for naming the root of the
+     directory whose VSM model you wish to construct.  The parameter 'want_stemming'
+     means that you would want the words in the documents to be stemmed to their root
+     forms before the VSM model is constructed.  Stemming will reduce all words such
+     as 'programming,' 'programs,' 'program,' etc. to the same root word 'program.'
+     The parameter 'break_camelcased_and_underscored' when set will cause all
+     underscored and camel-cased words to be split.  The constructor parameter
+     'min_word_length' causes all words to be discarded whose lengths are shorter
+     than the value given to this parameter.  The functions display_corpus_vocab()
+     and display_doc_vectors() are there only for testing purposes with small
+     corpora.  If you must use them for large libraries/corpora, you might wish to
+     redirect the output to a file.
 
-     By default, a call to any of the constructors will calculate
-     normalized term-frequency vectors for the documents.  Normalization
-     consists of first calculating the term frequency tf(t) of a term t in
-     a document as a proportion of the total numbers of words in the
-     document and then multiplying it by idf(t), where idf(t) stands for
-     the inverse document frequency associated with that term.  Note that
-     'word' and 'term' mean the same thing.
+     By default, a call to a constructor calculates normalized term-frequency vectors
+     for the documents.  Normalization consists of first calculating the term
+     frequency tf(t) of a term t in a document as a proportion of the total numbers
+     of words in the document and then multiplying it by idf(t), where idf(t) stands
+     for the inverse document frequency associated with that term.  Note that 'word'
+     and 'term' mean the same thing.
 
 
 
@@ -1256,13 +1241,11 @@ Analysis) algorithms in response to search words.
 
         my $lsa = Algorithm::VSM->new( 
                            corpus_directory         => $corpus_dir,
-                           corpus_vocab_db          => $corpus_vocab_db,
-                           doc_vectors_db           => $doc_vectors_db,
-                           normalized_doc_vecs_db   => $normalized_doc_vecs_db,
+                           use_idf_filter           => 1,
                            stop_words_file          => $stop_words_file,
                            want_stemming            => 1,
-                           save_model_on_disk       => 0,  
                            break_camelcased_and_underscored  => 1, 
+                           min_word_length          => 4,
                            lsa_svd_threshold        => 0.01, 
                            max_number_retrievals    => 10,
         );
@@ -1275,58 +1258,13 @@ Analysis) algorithms in response to search words.
         my $retrievals = $lsa->retrieve_for_query_with_lsa( \@query );
         $lsa->display_retrievals( $retrievals );
 
-    The initialization code before the constructor call and the calls for
-    displaying the vocabulary and the vectors after the call remain the
-    same as for the previous case.  In the calls above, the constructor
-    parameter lsa_svd_threshold determines how many of the singular values
-    will be retained after we have carried out an SVD decomposition of the
-    term-frequency matrix for the documents in the corpus.  Singular values
-    smaller than this threshold fraction of the largest value are rejected.
-    The parameters that end in '_db' are optional and are there for naming
-    the database files in which the basic information about the model is
-    stored when the parameter save_model_on_disk is set, as explained for
-    the previous constructor call.
-
-
-
-  # FOR USING A PREVIOUSLY CONSTRUCTED VSM MODEL FOR RETRIEVAL:
-
-        my @query = qw/ program ListIterator add ArrayList args /;
-        my $corpus_vocab_db = "corpus_vocab_db";
-        my $doc_vectors_db  = "doc_vectors_db";
-        my $normalized_doc_vecs_db  = "normalized_doc_vecs_db";
-        my $vsm = Algorithm::VSM->new( 
-                           corpus_vocab_db          => $corpus_vocab_db, 
-                           doc_vectors_db           => $doc_vectors_db,
-                           normalized_doc_vecs_db   => $normalized_doc_vecs_db,
-                           break_camelcased_and_underscored  => 1, 
-                           max_number_retrieval s   => 10,
-        );
-        $vsm->upload_normalized_vsm_model_from_disk();
-        my $retrievals = $vsm->retrieve_with_vsm( \@query );
-        $vsm->display_retrievals( $retrievals );
-
-    The code for displaying the various results after the constructor call
-    remains the same as in earlier examples.
-
-
-
-  # FOR USING A PREVIOUSLY CONSTRUCTED LSA MODEL FOR RETRIEVAL:
-
-        my $lsa = Algorithm::VSM->new( 
-                           corpus_vocab_db          => $corpus_vocab_db,
-                           doc_vectors_db           => $doc_vectors_db,
-                           normalized_doc_vecs_db   => $normalized_doc_vecs_db,
-                           break_camelcased_and_underscored  => 1, 
-                           max_number_retrievals    => 10,
-        );
-        $lsa->upload_normalized_vsm_model_from_disk();
-        $lsa->construct_lsa_model();
-        my $retrievals = $lsa->retrieve_with_lsa( \@query );
-        $lsa->display_retrievals( $retrievals );
-
-    The initialization code before the constructor call and the code for
-    displaying various entities remains the same as shown earlier.
+    The initialization code before the constructor call and the calls for displaying
+    the vocabulary and the vectors after the call remain the same as for the VSM case
+    shown previously in this Synopsis.  In the call above, the constructor parameter
+    'lsa_svd_threshold' determines how many of the singular values will be retained
+    after we have carried out an SVD decomposition of the term-frequency matrix for
+    the documents in the corpus.  Singular values smaller than this threshold
+    fraction of the largest value are rejected.
 
 
 
@@ -1341,7 +1279,8 @@ Analysis) algorithms in response to search words.
                            corpus_directory    => $corpus_dir,
                            stop_words_file     => $stop_words_file,
                            query_file          => $query_file,
-                           want_stemming       => 1,
+                           want_stemming       => 1
+                           min_word_length     => 4,
                            relevancy_threshold => 5, 
                            break_camelcased_and_underscored  => 1, 
                            relevancy_file      => $relevancy_file, 
@@ -1355,17 +1294,15 @@ Analysis) algorithms in response to search words.
         $vsm->display_precision_vs_recall_for_queries();
         $vsm->display_map_values_for_queries();
 
-      Measuring precision and recall requires a set of queries.  These are
-      supplied through the constructor parameter 'query_file'.  The format
-      of the this file must be according to the sample file
-      'test_queries.txt' in the 'examples' directory.  The module estimates
-      the relevancies of the documents to the queries and dumps the
-      relevancies in a file named by the 'relevancy_file' constructor
-      parameter.  The constructor parameter 'relevancy_threshold' is used
-      in deciding which of the documents are considered to be relevant to a
-      query.  A document must contain at least the 'relevancy_threshold'
-      occurrences of query words in order to be considered relevant to a
-      query.
+      Measuring precision and recall requires a set of queries.  These are supplied
+      through the constructor parameter 'query_file'.  The format of the this file
+      must be according to the sample file 'test_queries.txt' in the 'examples'
+      directory.  The module estimates the relevancies of the documents to the
+      queries and dumps the relevancies in a file named by the 'relevancy_file'
+      constructor parameter.  The constructor parameter 'relevancy_threshold' is used
+      to decide which of the documents are considered to be relevant to a query.  A
+      document must contain at least the 'relevancy_threshold' occurrences of query
+      words in order to be considered relevant to a query.
 
 
 
@@ -1379,6 +1316,7 @@ Analysis) algorithms in response to search words.
                            lsa_svd_threshold   => 0.01,
                            relevancy_threshold => 5,
                            break_camelcased_and_underscored  => 1, 
+                           min_word_length     => 4,
                            relevancy_file      => $relevancy_file,
         );
 
@@ -1391,17 +1329,16 @@ Analysis) algorithms in response to search words.
         $lsa->display_precision_vs_recall_for_queries();
         $lsa->display_map_values_for_queries();
 
-      We have already explained the purpose of the constructor parameter
-      'query_file' and about the constraints on the format of queries in
-      the file named through this parameter.  As mentioned earlier, the
-      module estimates the relevancies of the documents to the queries and
-      dumps the relevancies in a file named by the 'relevancy_file'
-      constructor parameter.  The constructor parameter
-      'relevancy_threshold' is used in deciding which of the documents are
-      considered to be relevant to a query.  A document must contain at
-      least the 'relevancy_threshold' occurrences of query words in order
-      to be considered relevant to a query.  We have previously explained
-      the role of the constructor parameter 'lsa_svd_threshold'.
+      We have already explained the purpose of the constructor parameter 'query_file'
+      and about the constraints on the format of queries in the file named through
+      this parameter.  As mentioned earlier, the module estimates the relevancies of
+      the documents to the queries and dumps the relevancies in a file named by the
+      'relevancy_file' constructor parameter.  The constructor parameter
+      'relevancy_threshold' is used in deciding which of the documents are considered
+      to be relevant to a query.  A document must contain at least the
+      'relevancy_threshold' occurrences of query words in order to be considered
+      relevant to a query.  We have previously explained the role of the constructor
+      parameter 'lsa_svd_threshold'.
 
 
 
@@ -1416,6 +1353,7 @@ Analysis) algorithms in response to search words.
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
+                   min_word_length     => 4,
                    break_camelcased_and_underscored  => 1, 
                    relevancy_file      => $relevancy_file,
         );
@@ -1427,12 +1365,11 @@ Analysis) algorithms in response to search words.
         $vsm->display_precision_vs_recall_for_queries();
         $vsm->display_map_values_for_queries();
 
-    Now the filename supplied through the constructor parameter
-    'relevancy_file' must contain relevance judgments for the queries that
-    are named in the file supplied through the parameter 'query_file'.  The
-    format of these two files must be according to what is shown in the
-    sample files 'test_queries.txt' and 'relevancy.txt' in the 'examples'
-    directory.
+    Now the filename supplied through the constructor parameter 'relevancy_file' must
+    contain relevance judgments for the queries that are named in the file supplied
+    through the parameter 'query_file'.  The format of these two files must be
+    according to what is shown in the sample files 'test_queries.txt' and
+    'relevancy.txt' in the 'examples' directory.
 
 
 
@@ -1444,11 +1381,10 @@ Analysis) algorithms in response to search words.
         my $relevancy_file   = "relevancy.txt";  
         my $lsa = Algorithm::VSM->new( 
                    corpus_directory    => $corpus_dir,
-                   corpus_vocab_db     => $corpus_vocab_db,
-                   doc_vectors_db      => $doc_vectors_db,
                    stop_words_file     => $stop_words_file,
                    query_file          => $query_file,
                    want_stemming       => 1,
+                   min_word_length     => 4,
                    lsa_svd_threshold   => 0.01,
                    break_camelcased_and_underscored  => 1, 
                    relevancy_file      => $relevancy_file,
@@ -1462,13 +1398,12 @@ Analysis) algorithms in response to search words.
         $lsa->display_precision_vs_recall_for_queries();
         $lsa->display_map_values_for_queries();
 
-    As mentioned for the previous code block, the filename supplied through
-    the constructor parameter 'relevancy_file' must contain relevance
-    judgments for the queries that are named in the file supplied through
-    the parameter 'query_file'.  The format of this file must be according
-    to what is shown in the sample file 'relevancy.txt' in the 'examples'
-    directory.  We have already explained the roles played by the
-    constructor parameters such as 'lsa_svd_threshold'.
+    As mentioned for the previous code block, the filename supplied through the
+    constructor parameter 'relevancy_file' must contain relevance judgments for the
+    queries that are named in the file supplied through the parameter 'query_file'.
+    The format of this file must be according to what is shown in the sample file
+    'relevancy.txt' in the 'examples' directory.  We have already explained the roles
+    played by the constructor parameters such as 'lsa_svd_threshold'.
 
 
 
@@ -1494,12 +1429,24 @@ Analysis) algorithms in response to search words.
 
 =head1 CHANGES
 
-Version 1.50 incorporates a couple of new features: (1) You now have the
-option to split camel-cased and underscored words for constructing your
-vocabulary set; and (2) Storing the VSM and LSA models in database files on
-the disk is now optional.  The second feature, in particular, should prove
-useful to those who are using this module for large collections of
-documents.
+Version 1.60 reflects the fact that people are now more likely to use this module by
+keeping the model constructed for a corpus in the fast memory (as opposed to storing
+the models in disk-based hash tables) for its repeated invocation for different
+queries.  As a result, the default value for the constructor option
+C<save_model_on_disk> was changed from 1 to 0.  For those who still wish to store on
+a disk the model that is constructed, the script
+C<retrieve_with_VSM_and_also_create_disk_based_model.pl> shows how you can do that.
+Other changes in 1.60 include a slight reorganization of the scripts in the
+C<examples> directory.  Most scripts now do not by default store their models in
+disk-based hash tables.  This reorganization is reflected in the description of the
+C<examples> directory in this documentation.  The basic logic of constructing VSM and
+LSA models and how these are used for retrievals remains unchanged.
+
+Version 1.50 incorporates a couple of new features: (1) You now have the option to
+split camel-cased and underscored words for constructing your vocabulary set; and (2)
+Storing the VSM and LSA models in database files on the disk is now optional.  The
+second feature, in particular, should prove useful to those who are using this module
+for large collections of documents.
 
 Version 1.42 includes two new methods, C<display_corpus_vocab_size()> and
 C<write_corpus_vocab_to_file()>, for those folks who deal with very large datasets.
@@ -1670,7 +1617,7 @@ A call to C<new()> constructs a new instance of the C<Algorithm::VSM> class:
                      query_file             => "",  
                      relevancy_file         => $relevancy_file,
                      relevancy_threshold    => 5, 
-                     save_model_on_disk     => 1,  
+                     save_model_on_disk     => 0,  
                      stop_words_file        => "", 
                      use_idf_filter         => 1,
                      want_stemming          => 1,
@@ -2086,34 +2033,53 @@ For basic VSM-based model construction and retrieval, run the script:
 
     retrieve_with_VSM.pl
 
+Starting with version 1.60, this script does not store away the VSM model in
+disk-based hash tables.  If you want your model to be stored on the disk, you must
+run the script C<retrieve_with_VSM_and_also_create_disk_based_model.pl> for that.
+
+=item B<For a Continuously Running VSM-Based Search Engine for Repeated Retrievals:>
+
+If you want to run an infinite loop for repeated retrievals from a VSM model, run the
+script
+
+    continuously_running_VSM_retrieval_engine.pl
+
+You can create a script similar to this for doing the same with LSA models.
+
+=item B<For Storing the Model Information in Disk-Based Hash Tables:>
+
+For storing the model information in disk-based DBM files that can subsequently be
+used for both VSM and LSA retrieval, run the script:
+
+    retrieve_with_VSM_and_also_create_disk_based_model.pl
+
 =item B<For Basic LSA-Based Retrieval:>
 
 For basic LSA-based model construction and retrieval, run the script:
 
     retrieve_with_LSA.pl
 
-Both of the above scripts will store the corpus models created in disk-based
-databases.
+Starting with version 1.60, this script does not store away the model information in
+disk-based hash tables.  If you want your model to be stored on the disk, you must
+run the script C<retrieve_with_VSM_and_also_create_disk_based_model.pl> for that.
 
 =item B<For VSM-Based Retrieval with a Disk-Stored Model:>
 
-If you have previously run a script like C<retrieve_with_VSM.pl> and no intervening
-code has modified the disk-stored VSM model of the corpus, you can run the script
+If you have previously run a script like
+C<retrieve_with_VSM_and_also_create_disk_based_model.pl>, you can run the script
 
     retrieve_with_disk_based_VSM.pl
 
-This would obviously work faster at retrieval since the VSM model would NOT need to
-constructed for each new query.
+for repeated VSM-based retrievals from a disk-based model.
 
 =item B<For LSA-Based Retrieval with a Disk-Stored Model:>
 
-If you have previously run a script like C<retrieve_with_LSA.pl> and no intervening
-code has modified the disk-stored LSA model of the corpus, you can run the script
+If you have previously run a script like
+C<retrieve_with_VSM_and_also_create_disk_based_model.pl>, you can run the script
 
     retrieve_with_disk_based_LSA.pl
 
-The retrieval performance of such a script would be faster since the LSA model would
-NOT need to be constructed for each new query.
+for repeated LSA-based retrievals from a disk-based model.
 
 =item B<For Precision and Recall Calculations with VSM:>
 
@@ -2135,7 +2101,6 @@ script:
 Note that this script will carry out its own estimation of relevancy judgments ---
 which in most cases would not be a safe thing to do.
 
-
 =item B<For Precision and Recall Calculations for VSM with
 Human-Supplied Relevancies:>
 
@@ -2212,10 +2177,10 @@ the string 'VSM' in the subject line to get past my spam filter.
 Download the archive from CPAN in any directory of your choice.  Unpack the archive
 with a command that on a Linux machine would look like:
 
-    tar zxvf Algorithm-VSM-1.50.tar.gz
+    tar zxvf Algorithm-VSM-1.60.tar.gz
 
 This will create an installation directory for you whose name will be
-C<Algorithm-VSM-1.50>.  Enter this directory and execute the following commands for a
+C<Algorithm-VSM-1.60>.  Enter this directory and execute the following commands for a
 standard install of the module if you have root privileges:
 
     perl Makefile.PL
@@ -2248,7 +2213,9 @@ If I used bash, I'd need to declare:
 
 Many thanks are owed to Shivani Rao and Bunyamin Sisman for sharing with me their
 deep insights in IR.  Version 1.4 was prompted by Zahn Bozanic's interest in
-similarity matrix characterization of a corpus. Thanks, Zahn!
+similarity matrix characterization of a corpus. Thanks, Zahn!  Several of the recent
+changes to the module are a result of the feedback I have received from Naveen
+Kulkarni of Infosys Labs. Thanks, Naveen!
 
 =head1 AUTHOR
 
@@ -2262,7 +2229,7 @@ spam filter.
 This library is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
 
- Copyright 2014 Avinash Kak
+ Copyright 2015 Avinash Kak
 
 =cut
 
